/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: cimath.hpp,v 1.6 2006/11/24 10:00:58 cxsc Exp $ */

/*
**
**  COmplex interval STandard functions LibrarY, CoStLy Version 1.0.3
**
**  Copyright (C) Markus Neher,        markus.neher@math.uni-karlsruhe.de
**                Ingo Eble,           ingoeble@web.de
**                Frithjof Blomquist,  Blomquist@math.uni-wuppertal.de
**
**  The complex interval elementary functions in C-XSC are based on the 
**  CoStLy library written by Markus Neher. Additional improvements have
**  been done by Frithjof Blomquist.
**  
**  References:
**  - Neher, M: "Complex Standard Functions and their Implementation in
**    the CoStLy Library", Preprint Nr. 04/18, Fakultaet fuer Mathematik,
**    Universitaet Karlsruhe, 2004.
**  - Blomquist, F.; Hofschuster, W.; Kraemer, W.: "Complex Interval Functions 
**    in C-XSC", Preprint BUW-WRSWT 2005/2, Universitaet Wuppertal, 2005.
**  - Neher, M: "Complex Standard Functions and their Implementation in
**    the CoStLy Library", to appear in ACM Transactions on Mathematical 
**    Software (TOMS).
**
*/


#ifndef _CIMATH_HPP_INCLUDED
#define _CIMATH_HPP_INCLUDED

#include "cinterval.hpp"
#include <list>
#include <string>
#include <cmath>

namespace cxsc{

cinterval exp(const cinterval&);
cinterval cos(const cinterval&);
cinterval sin(const cinterval&);
cinterval cosh(const cinterval&);
cinterval sinh(const cinterval&);

cinterval tan(const cinterval&) throw();
cinterval cot(const cinterval&);
cinterval tanh(const cinterval&);
cinterval coth(const cinterval&);

interval arg(const cinterval&);
interval arg_inclmon(const cinterval&);
interval Arg(const cinterval&) throw();

cinterval ln(const cinterval&) throw();
cinterval Ln(const cinterval&) throw();

cinterval sqr(const cinterval&);

cinterval sqrt(const cinterval&) throw();
std::list<cinterval>sqrt_all(const cinterval&);
cinterval sqrt(const cinterval&, int) throw();
std::list<cinterval>sqrt_all(const cinterval&, int);

cinterval power_fast(const cinterval&,int) throw();
cinterval power(const cinterval&,int) throw();
cinterval pow(const cinterval&, const interval&) throw();
cinterval pow(const cinterval&, const cinterval&) throw();
std::list<cinterval>pow_all(const cinterval&, const interval&) throw();
void times2pown(cinterval& x, int n) throw();

cinterval asin(const cinterval&) throw();
cinterval acos(const cinterval&);
cinterval asinh(const cinterval&);
cinterval acosh(const cinterval&) throw();
cinterval atan(const cinterval&) throw();
cinterval acot(const cinterval&) throw();
cinterval atanh(const cinterval&);
cinterval acoth(const cinterval&);

} // namespace cxsc

#endif

/*

  End of File: cimath.hpp

*/

