/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: imath.hpp,v 1.18 2006/12/01 11:39:31 cxsc Exp $ */

#ifndef imath_hpp_included
#define imath_hpp_included

#include "except.hpp"
#include "interval.hpp"

namespace cxsc {

interval sqr     (const interval&) throw(); // sqr(x)
interval sqrt    (const interval&);         // sqrt(x)
interval sqrt    (const interval &, int) throw(STD_FKT_OUT_OF_DEF);  // sqrt(x, n)
interval sqrt1px2(const interval&) throw(); // sqrt(1+x^2)
interval sqrtp1m1(const interval&) throw(); // sqrt(x+1)-1
interval sqrtx2m1(const interval&);         // sqrt(x^2-1)
interval sqrt1mx2(const interval&);         // sqrt(1-x^2)

interval sin     (const interval&) throw(); // sin(x)
interval cos     (const interval&) throw(); // cos(x)
interval tan     (const interval&) throw(); // tan(x)
interval cot     (const interval&) throw(); // cot(x)

interval asin    (const interval&);         // ASin(x)
interval acos    (const interval&);         // ACos(x)
interval atan    (const interval&);         // ATan(x) 
interval acot    (const interval&);         // ACot(x)

interval exp     (const interval&) throw(); // exp(x)
interval expmx2  (const interval&);         // exp(-x^2)
interval expm1   (const interval&);         // exp(x)-1
interval expx2   (const interval& x);       // e^{+x^2}
interval expx2m1 (const interval& x);       // e^{+x^2}-1
interval ln      (const interval&);         // ln(x)
interval lnp1    (const interval&) throw(); // ln(1+x)

interval sinh    (const interval&) throw(); // Sinh(x)
interval cosh    (const interval&) throw(); // Cosh(x)
interval tanh    (const interval&) throw(); // Tanh(x) 
interval coth    (const interval&) throw(); // Coth(x)

interval asinh   (const interval&);         // ASinh(x)
interval acosh   (const interval&);         // ACosh(x)
interval acoshp1 (const interval&);         // acosh(1+x)
interval atanh   (const interval&);         // ATanh(x)
interval acoth   (const interval&);         // ACoth(x)

interval erf     (const interval&);         // error function
interval erfc    (const interval&);         // complementary error function

interval pow     (const interval&, const interval&) throw();  // Pow(x,y)
interval power   (const interval&, int);    // Power(x,n)
interval Power   (const interval&, int);    // Power(x,n)

interval sqrtx2y2(const interval&, const interval&) throw(); // sqrt(x^2+y^2)
interval ln_sqrtx2y2(const interval&, const interval&) throw();
                                            // ln( sqrt(x^2+y^2) ) == 0.5*ln(x^2+y^2);
interval Pi         ( );
} // namespace cxsc 

// #include "imath.inl"

#endif

